/***************************************************************************
    qgsvectorlayerref.h
    ---------------------
    begin                : January 2017
    copyright            : (C) 2017 by Martin Dobias
    email                : wonder dot sk at gmail dot com
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef QGSVECTORLAYERREF_H
#define QGSVECTORLAYERREF_H

#define SIP_NO_FILE

#include "qgsmaplayerref.h"
#include "qgsvectorlayer.h"

typedef _LayerRef<QgsVectorLayer> QgsVectorLayerRef;

#endif // QGSVECTORLAYERREF_H
